srdusr
aboutsummaryrefslogtreecommitdiffstats
path: root/internal/ipc/server.go
diff options
context:
space:
mode:
Diffstat (limited to 'internal/ipc/server.go')
-rw-r--r--internal/ipc/server.go44
1 files changed, 42 insertions, 2 deletions
diff --git a/internal/ipc/server.go b/internal/ipc/server.go
index 1fe8d8f..50dff7f 100644
--- a/internal/ipc/server.go
+++ b/internal/ipc/server.go
@@ -18,6 +18,13 @@ type Repeater interface {
Repeat(ctx context.Context, scheme, host string, raw []byte) (*store.Entry, error)
}
+// Intruder runs a Sniper attack over a §marked§ request template -
+// implemented by *proxy.Server.
+type Intruder interface {
+ Intrude(ctx context.Context, scheme, host string, template []byte, payloads []string,
+ onResult func(position int, payload string, entry *store.Entry, sendErr error) bool) error
+}
+
// Hub fans out newly captured history entries to subscribed clients.
type Hub struct {
mu sync.Mutex
@@ -62,12 +69,17 @@ type Server struct {
db *store.Store
hub *Hub
repeater Repeater
+ intruder Intruder
}
// NewServer creates a control-protocol Server backed by db, broadcasting
// through hub and sending Repeater requests through rep.
func NewServer(db *store.Store, hub *Hub, rep Repeater) *Server {
- return &Server{db: db, hub: hub, repeater: rep}
+ s := &Server{db: db, hub: hub, repeater: rep}
+ if in, ok := rep.(Intruder); ok {
+ s.intruder = in
+ }
+ return s
}
// Serve accepts connections on ln until it returns an error (e.g. the
@@ -128,6 +140,34 @@ func (s *Server) handleConn(conn net.Conn) {
}
enc.Encode(Response{Type: "repeat", Detail: detailFromEntry(e)})
+ case "intrude":
+ if s.intruder == nil {
+ enc.Encode(Response{Type: "error", Error: "intruder not available"})
+ continue
+ }
+ err := s.intruder.Intrude(context.Background(), req.Scheme, req.Host, req.Raw, req.Payloads,
+ func(position int, payload string, entry *store.Entry, sendErr error) bool {
+ r := IntrudeResultMsg{Position: position, Payload: payload}
+ if sendErr != nil {
+ r.Error = sendErr.Error()
+ }
+ if entry != nil {
+ r.EntryID = entry.ID
+ r.StatusCode = entry.StatusCode
+ r.RespSize = len(entry.ResponseRaw)
+ r.Duration = entry.Duration
+ if entry.Error != "" && r.Error == "" {
+ r.Error = entry.Error
+ }
+ }
+ return enc.Encode(Response{Type: "intrude_result", IntrudeResult: &r}) == nil
+ })
+ if err != nil {
+ enc.Encode(Response{Type: "error", Error: err.Error()})
+ continue
+ }
+ enc.Encode(Response{Type: "intrude_done"})
+
case "rules_list":
rs, err := s.db.ListRules()
if err != nil {
@@ -62,12 +69,17 @@ type Server struct {
}
// NewServer creates a control-protocol Server backed by db, broadcasting
-// through hub and sending Repeater requests through rep.
+// through hub and sending Repeater/Intruder requests through rep.
func NewServer(db *store.Store, hub *Hub, rep Repeater) *Server {